@@ -1,7 +1,7 @@
 AS.select <-
 function (obs, param, sumstats, obspar = NULL, abcmethod = abc, 
     grid = 10, inturn = TRUE, limit = ncol(sumstats), allow.none = FALSE, 
-    do.err = FALSE, final.dens = FALSE, errfn = rsse, ...) 
+    do.err = FALSE, final.dens = FALSE, errfn = rsse, trace=TRUE, ...) 
 {
 
 argl <- list(...)
@@ -52,14 +52,14 @@ if ((length(margind)==0)&identical(abcmethod,abc)){
     nstats <- ncol(sumstats)
     limit <- min(nstats, limit)
     order <- sample(1:nstats, limit, F)
-    cat("order is:", order, "\n")
+    if (trace) cat("Sumstat order for testing is:", order, "\n")
     I <- numeric(0)
     data <- obs
     argl$param=param
 
     for (j in 1:length(order)) {
-        cat("I is:", I, ", testing:", c(I, order[j]), "\n")
         if ((length(I) == 0)) {
+            if (trace) cat("Current subset is: empty Test adding:", order[j], "\n")
             if (allow.none) {
                 index1 <- sample(param, ceiling(length(param) * 
                   eps), T)
@@ -74,12 +74,21 @@ if ((length(margind)==0)&identical(abcmethod,abc)){
                   index2 <- index2$adj.values
                 }
                 add <- AS.test(grid, index1, index2, supp)
+                if (trace) {
+                    if (add) {
+                        cat("Significant change to ABC posterior - add\n\n")
+                    } else {
+                        cat("No significant change to ABC posterior - don't add\n\n")
+                    }
+                }
             }
             else {
                 add <- TRUE
+                if (trace) cat("Empty subset not allowed - add \n\n")
             }
         }
         else {
+            if (trace) cat("Current subset is:", I, " Test adding:", order[j], "\n")
 		argl$target=data[I]
 		argl$sumstat=sumstats[,I]
 		index1 <-do.call(abcmethod,argl)
@@ -99,22 +108,32 @@ if ((length(margind)==0)&identical(abcmethod,abc)){
                 index2 <- index2$adj.values
             }
             add <- AS.test(grid, index1, index2, supp)
+            if (trace) {
+                if (add) {
+                    cat("Significant change to ABC posterior - add \n\n")
+                } else {
+                    cat("No significant change to ABC posterior - don't add \n\n")
+                }
+            }
         }
         if (add) {
             I <- c(I, order[j])
             bad <- NULL
             if (length(I) > 1) {
+                if (trace) cat("Consider removing previous summaries\n")
+                if (trace) cat("Current subset is:", I, " ")
                 if (inturn) {
                   for (i in 1:(length(I) - 1)) {
+                    if (trace) cat("Test removing:", I[i], "\n")
                     subset2 <- I
                     subset1 <- setdiff(subset2, I[i])
                     if (length(subset1) == 0) {
                       if (allow.none) {
                         i1 <- sample(param, ceiling(length(param) * 
                           eps), T)
-		argl$target=data[subset2]
-		argl$sumstat=sumstats[,subset2]
-		i2<-do.call(abcmethod,argl)
+                        argl$target=data[subset2]
+                        argl$sumstat=sumstats[,subset2]
+                        i2<-do.call(abcmethod,argl)
                         if (is.null(i2$adj.values)) {
                           i2 <- i2$unadj.values
                         }
@@ -122,11 +141,19 @@ if ((length(margind)==0)&identical(abcmethod,abc)){
                           i2 <- i2$adj.values
                         }
                         add2 <- AS.test(grid, i1, i2, supp)
+                        if (trace) {
+                            if (add2) {
+                                cat("Significant change to ABC posterior - keep\n\n")
+                            } else {
+                                cat("No significant change to ABC posterior - remove\n\n")
+                            }
+                        }                        
                       }
                       else {
                         add2 <- TRUE
+                        if (trace) cat("Empty subset not allowed - keep\n\n")
                       }
-                    }
+                  }
                     else {
 			argl$target=data[subset1]
 			argl$sumstat=sumstats[,subset1]
@@ -148,6 +175,13 @@ if ((length(margind)==0)&identical(abcmethod,abc)){
                         i2 <- i2$adj.values
                       }
                       add2 <- AS.test(grid, i1, i2, supp)
+                      if (trace) {
+                          if (add2) {
+                              cat("Significant change to ABC posterior - keep\n\n")
+                          } else {
+                              cat("No significant change to ABC posterior - remove\n\n")
+                          }
+                      } 
                     }
                     if (add2) {
                       I <- subset2
@@ -159,6 +193,7 @@ if ((length(margind)==0)&identical(abcmethod,abc)){
                 }
                 else {
                   for (i in 1:(length(I) - 1)) {
+                    if (trace) cat("Test removing:", I[i], "\n")
                     subset2 <- I
                     subset1 <- setdiff(subset2, I[i])
                     if (length(subset1) == 0) {
@@ -175,8 +210,14 @@ if ((length(margind)==0)&identical(abcmethod,abc)){
                           i2 <- i2$adj.values
                         }
                         add2 <- AS.test(grid, i1, i2, supp)
+                        if (add2) {
+                            cat("Significant change to ABC posterior - keep\n\n")
+                        } else {
+                            cat("No significant change to ABC posterior - remove\n\n")
+                        }                        
                       }
                       else {
+                        if (trace) cat("Empty subset not allowed - keep\n\n")
                         add2 <- TRUE
                       }
                     }
@@ -201,16 +242,23 @@ if ((length(margind)==0)&identical(abcmethod,abc)){
                         i2 <- i2$adj.values
                       }
                       add2 <- AS.test(grid, i1, i2, supp)
+                      if (add2) {
+                          cat("Significant change to ABC posterior - keep\n\n")
+                      } else {
+                          cat("No significant change to ABC posterior - remove\n\n")
+                      } 
                     }
                     if (!add2) {
                       bad <- c(bad, I[i])
                     }
                   }
+                  if (trace) cat("Removing unnecessary summaries:", bad, "\n\n")
                   I <- setdiff(I, bad)
                 }
             }
         }
     }
+    if (trace) cat("Selected summaries:", I, "\n")
     l <- list()
     best <- I
     if ((length(I) == 0)) {
